package izziv2;

public class Izziv2 {
    public static void main(String[] args) throws CollectionException {
        System.out.println("Stack: ");
        Stack<Integer> stack = new ArrayDeque<>();

        int i = 0;
        while (!stack.isFull()) {
            stack.push(i++);
        }
        System.out.println(stack);

        assert stack.top() == i;

        while (!stack.isEmpty()) {
            System.out.println(stack.pop());
        }

        System.out.println("Deque: ");
        Deque<Integer> deque = new ArrayDeque<>();

        // enqueue and dequeueBack are the same as push and pop
        i = 0;
        while (!deque.isFull()) {
            deque.enqueueFront(i++);
        }
        System.out.println(deque);

        assert deque.front() == i;
        assert deque.back() == 0;

        while (!deque.isEmpty()) {
            System.out.println(deque.dequeue());
        }

        System.out.println("Sequence: ");
        Sequence<Integer> sequence = new ArrayDeque<>();

        i = 0;
        while (!sequence.isFull()) {
            sequence.add(i++);
        }
        System.out.println(sequence);
        for (int j = 0; j < sequence.size(); j++) {
            System.out.println(sequence.get(j));
        }
    }
}

class CollectionException extends Exception {
    public CollectionException(String msg) {
        super(msg);
    }
}

interface Collection {
    static final String ERR_MSG_EMPTY = "Collection is empty.";
    static final String ERR_MSG_FULL = "Collection is full.";

    boolean isEmpty();

    boolean isFull();

    int size();

    String toString();
}

interface Stack<T> extends Collection {
    T top() throws CollectionException;

    void push(T x) throws CollectionException;

    T pop() throws CollectionException;
}

interface Deque<T> extends Collection {
    T front() throws CollectionException;

    T back() throws CollectionException;

    void enqueue(T x) throws CollectionException;

    void enqueueFront(T x) throws CollectionException;

    T dequeue() throws CollectionException;

    T dequeueBack() throws CollectionException;
}

interface Sequence<T> extends Collection {
    static final String ERR_MSG_INDEX = "Wrong index in sequence.";

    T get(int i) throws CollectionException;

    void add(T x) throws CollectionException;
}

class ArrayDeque<T> implements Deque<T>, Stack<T>, Sequence<T> {
    private static final int DEFAULT_CAPACITY = 64;

    private final T[] array;
    private int front, back, size;

    public ArrayDeque() {
        //noinspection unchecked
        array = (T[]) new Object[DEFAULT_CAPACITY];
        front = back = size = 0;
    }

    private int index(int i) {
        return (front + i) % DEFAULT_CAPACITY;
    }

    private int next(int i) {
        return (i + 1) % DEFAULT_CAPACITY;
    }

    private int prev(int i) {
        return (i + DEFAULT_CAPACITY - 1) % DEFAULT_CAPACITY;
    }

    @Override
    public T front() throws CollectionException {
        if (isEmpty()) {
            throw new CollectionException(ERR_MSG_EMPTY);
        }
        return array[front];
    }

    @Override
    public T back() throws CollectionException {
        if (isEmpty()) {
            throw new CollectionException(ERR_MSG_EMPTY);
        }
        return array[back];
    }

    @Override
    public void enqueue(T x) throws CollectionException {
        this.push(x);
    }

    @Override
    public void enqueueFront(T x) throws CollectionException {
        if (isFull()) {
            throw new CollectionException(ERR_MSG_FULL);
        }

        front = this.prev(front);
        size++;

        array[front] = x;
    }

    @Override
    public T dequeue() throws CollectionException {
        if (isEmpty()) {
            throw new CollectionException(ERR_MSG_EMPTY);
        }

        T x = array[front];
        array[front] = null;
        front = this.next(front);
        size--;
        return x;
    }

    @Override
    public T dequeueBack() throws CollectionException {
        return this.pop();
    }

    @Override
    public T get(int i) throws CollectionException {
        if (isEmpty()) {
            throw new CollectionException(ERR_MSG_EMPTY);
        }

        if (i < 0 || i >= size) {
            throw new CollectionException(ERR_MSG_INDEX);
        }

        return array[index(i)];
    }

    @Override
    public void add(T x) throws CollectionException {
        this.push(x);
    }

    @Override
    public T top() throws CollectionException {
        return this.back();
    }

    @Override
    public void push(T x) throws CollectionException {
        if (isFull()) {
            throw new CollectionException(ERR_MSG_FULL);
        }

        array[back] = x;
        back = this.next(back);
        size++;
    }

    @Override
    public T pop() throws CollectionException {
        if (isEmpty()) {
            throw new CollectionException(ERR_MSG_EMPTY);
        }

        back = this.prev(back);
        T x = array[back];
        array[back] = null;
        size--;
        return x;
    }

    @Override
    public boolean isEmpty() {
        return size == 0;
    }

    @Override
    public boolean isFull() {
        return size == DEFAULT_CAPACITY;
    }

    @Override
    public int size() {
        return size;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("[");
        if (size > 0) {
            sb.append(array[front].toString());
        }
        for (int i = 0; i < size - 1; i++) {
            sb.append(", ").append(array[this.next(front + i)].toString());
        }
        sb.append("]");
        return sb.toString();
    }
}