import java.util.Scanner;

public class Naloga2 {
    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);

        String[] settings = scanner.nextLine().split("\\s+");
        boolean up = settings[2].equals("up");

        String[] data = scanner.nextLine().split("\\s+");
        IntArray array = new IntArray(data.length);
        for (String s : data) {
            array.append(Integer.parseInt(s));
        }

        Sorter sorter;

        switch (settings[1]) {
            case "insert":
                sorter = new InsertionSorter();
                break;
            case "select":
                sorter = new SelectionSorter();
                break;
            case "bubble":
                sorter = new BubbleSorter();
                break;
            case "heap":
                sorter = new HeapSorter();
                break;
            case "merge":
                sorter = new MergeSorter();
                break;
            case "quick":
                sorter = new QuickSorter();
                break;
            case "radix":
                sorter = new RadixSorter();
                break;
            case "bucket":
                sorter = new BucketSorter();
                break;
            default:
                throw new Error("Invalid sort type");
        }

        switch (settings[0]) {
            case "trace":
                sorter.trace(array, up);
                break;
            case "count":
                sorter.count(array, up);
                break;
            default:
                throw new Error("Invalid output type");
        }
    }
}

class IntArray {
    private int[] array;
    private int size = 0;

    IntArray() {
        array = new int[64];
    }

    IntArray(int size) {
        array = new int[size];
    }

    private void resize() {
        int[] tmp = new int[array.length * 2];
        System.arraycopy(array, 0, tmp, 0, array.length);
        array = tmp;
    }

    private boolean isFull() {
        return size == array.length;
    }

    boolean isEmpty() {
        return size == 0;
    }

    int size() {
        return size;
    }

    void append(int element) {
        if (isFull()) {
            resize();
        }
        array[size++] = element;
    }

    int get(int index) {
        return array[index];
    }

    void set(int index, int element) {
        array[index] = element;
    }

    void swap(int index1, int index2) {
        int temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    }

    IntArray slice(int start, int end) {
        IntArray result = new IntArray();
        for (int i = start; i < end; i++) {
            result.append(array[i]);
        }
        return result;
    }

    void fill(int count, int value) {
        size = 0;
        for (int i = 0; i < count; i++) {
            append(value);
        }
    }

    public String toStringSplit(int[] splits) {
        return toStringSplit(splits, 0, size());
    }

    public String toStringSplit(int[] splits, int start, int end) {
        if (isEmpty()) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        for (int split : splits) {
            if (split == start) {
                sb.append("| ");
            }
        }
        sb.append(array[start]);
        for (int i = start + 1; i < end; i++) {
            for (int split : splits) {
                if (i == split) {
                    sb.append(" |");
                }
            }
            sb.append(" ").append(array[i]);
        }
        for (int split : splits) {
            if (end == split) {
                sb.append(" |");
            }
        }
        return sb.toString();
    }

    @Override
    public String toString() {
        if (isEmpty()) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        sb.append(array[0]);
        for (int i = 1; i < size; i++) {
            sb.append(" ").append(array[i]);
        }
        return sb.toString();
    }
}


abstract class Sorter {
    int moves = 0;
    int comparisons = 0;
    boolean trace = false;

    void sort(IntArray array, boolean up) {
        moves = 0;
        comparisons = 0;
        printTrace(array + "\n");
    }

    void trace(IntArray array, boolean up) {
        trace = true;
        sort(array, up);
    }

    void count(IntArray array, boolean up) {
        trace = false;
        sort(array, up);
        System.out.print(moves + " " + comparisons);
        sort(array, up);
        System.out.print(" | " + moves + " " + comparisons);
        sort(array, !up);
        System.out.println(" | " + moves + " " + comparisons);
    }

    void printTrace(Object value) {
        if (trace) {
            System.out.print(value);
        }
    }
}

class InsertionSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);

        for (int i = 1; i < array.size(); i++) {
            moves++;
            int tmp = array.get(i);
            int j = i;
            while (j > 0) {
                comparisons++;
                if (!(up ? array.get(j - 1) > tmp : array.get(j - 1) < tmp)) {
                    break;
                }
                moves++;
                array.set(j, array.get(j - 1));
                j--;
            }
            moves++;
            array.set(j, tmp);

            printTrace(array.toStringSplit(new int[]{i + 1}) + "\n");
        }
    }
}

class SelectionSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);

        for (int i = 0; i < array.size() - 1; i++) {
            int m = i;
            for (int j = i + 1; j < array.size(); j++) {
                comparisons++;
                if (up ? array.get(j) < array.get(m) : array.get(j) > array.get(m)) {
                    m = j;
                }
            }
            moves += 3;
            array.swap(i, m);

            printTrace(array.get(0));
            for (int l = 1; l < array.size(); l++) {
                if (l - 1 == i) {
                    printTrace(" |");
                }
                printTrace(" " + array.get(l));
            }
            printTrace("\n");
        }
    }
}

class BubbleSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);

        for (int i = 1; i < array.size(); i++) {
            int swap = array.size() - 1;
            for (int j = array.size() - 1; j >= i; j--) {
                comparisons++;
                if (up ? array.get(j - 1) > array.get(j) : array.get(j - 1) < array.get(j)) {
                    array.swap(j - 1, j);
                    swap = j;
                    moves += 3;
                }
            }

            i = swap;

            if (i == 0) {
                continue;
            }
            printTrace(array.get(0));
            for (int l = 1; l < array.size(); l++) {
                if (l == i) {
                    printTrace(" |");
                }
                printTrace(" " + array.get(l));
            }
            printTrace("\n");
        }
    }
}

class HeapSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);

        for (int i = array.size() / 2 - 1; i >= 0; i--) {
            siftDown(array, array.size(), i, up);
        }

        printTrace(array + " |\n");

        for (int i = array.size() - 1; i > 0; i--) {
            array.swap(0, i);
            moves += 3;
            siftDown(array, i, 0, up);

            printTrace(array.get(0));
            for (int l = 1; l < array.size(); l++) {
                if (l == i) {
                    printTrace(" |");
                }
                printTrace(" " + array.get(l));
            }
            printTrace("\n");
        }
    }

    void siftDown(IntArray array, int n, int i, boolean up) {
        int extr = i;
        int left = 2 * i + 1;
        int right = 2 * i + 2;

        if (left < n) {
            comparisons++;
            if (up ? array.get(left) > array.get(extr) : array.get(left) < array.get(extr)) {
                extr = left;
            }
        }

        if (right < n) {
            comparisons++;
            if (up ? array.get(right) > array.get(extr) : array.get(right) < array.get(extr)) {
                extr = right;
            }
        }

        if (extr != i) {
            array.swap(extr, i);
            moves += 3;
            siftDown(array, n, extr, up);
        }
    }
}

class MergeSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);
        mergeSort(array, up);
    }

    IntArray mergeSort(IntArray array, boolean up) {
        if (array.size() <= 1) {
            return array;
        }
        int middle = (array.size() - 1) / 2 + 1;

        IntArray left = array.slice(0, middle);
        moves += left.size();
        IntArray right = array.slice(middle, array.size());
        moves += right.size();
        printTrace(left + " | " + right + "\n");

        left = mergeSort(left, up);
        right = mergeSort(right, up);

        IntArray merged = merge(left, right, up);
        printTrace(merged + "\n");

        return merged;
    }

    IntArray merge(IntArray left, IntArray right, boolean up) {
        IntArray result = new IntArray();

        int i = 0;
        int j = 0;

        while (i < left.size() && j < right.size()) {
            comparisons++;
            if (up ? left.get(i) <= right.get(j) : left.get(i) >= right.get(j)) {
                result.append(left.get(i));
                i++;
            } else {
                result.append(right.get(j));
                j++;
            }
            moves++;
        }

        while (i < left.size()) {
            result.append(left.get(i));
            moves++;
            i++;
        }

        while (j < right.size()) {
            result.append(right.get(j));
            moves++;
            j++;
        }

        return result;
    }
}

class QuickSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);
        quickSort(array, 0, array.size() - 1, up);
        printTrace(array + "\n");
    }

    void quickSort(IntArray array, int left, int right, boolean up) {
        if (left >= right) {
            return;
        }
        int r = partition(array, left, right, up);

        printTrace(array.toStringSplit(new int[]{r, r + 1}, left, right + 1) + "\n");

        quickSort(array, left, r - 1, up);
        quickSort(array, r + 1, right, up);
    }

    int partition(IntArray array, int left, int right, boolean up) {
        int pivot = array.get(left);
        moves++;
        int l = left;
        int r = right + 1;
        while (true) {
            do {
                l++;
                comparisons++;
            } while ((up ? array.get(l) < pivot : array.get(l) > pivot) && l < right);
            do {
                r--;
                comparisons++;
            } while (up ? array.get(r) > pivot : array.get(r) < pivot);
            if (l >= r) {
                break;
            }
            array.swap(l, r);
            moves += 3;
        }
        array.swap(left, r);
        moves += 3;
        return r;
    }
}

class RadixSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);

        int m = getMax(array);

        for (int exp = 1; m / exp > 0; exp *= 10) {
            countSort(array, exp, up);
            printTrace(array + "\n");
        }
    }

    int getMax(IntArray array) {
        int max = array.get(0);
        for (int i = 1; i < array.size(); i++) {
            if (array.get(i) > max) {
                max = array.get(i);
            }
        }
        return max;
    }

    void countSort(IntArray array, int exp, boolean up) {
        IntArray output = new IntArray(array.size());
        int i;
        IntArray count = new IntArray();
        count.fill(10, 0);

        for (i = 0; i < array.size(); i++) {
            int c = (array.get(i) / exp) % 10;
            count.set(c, count.get(c) + 1);
            moves++;
            comparisons++;
        }

        if (up) {
            for (i = 1; i < 10; i++) {
                count.set(i, count.get(i) + count.get(i - 1));
            }
        } else {
            for (i = 8; i >= 0; i--) {
                count.set(i, count.get(i) + count.get(i + 1));
            }
        }

        for (i = array.size() - 1; i >= 0; i--) {
            int c = (array.get(i) / exp) % 10;
            output.set(count.get(c) - 1, array.get(i));
            count.set(c, count.get(c) - 1);
            moves++;
            comparisons++;
        }

        for (i = 0; i < array.size(); i++) {
            array.set(i, output.get(i));
        }
    }
}

class BucketSorter extends Sorter {
    @Override
    void sort(IntArray array, boolean up) {
        super.sort(array, up);
        int min = getMin(array);
        int max = getMax(array);

        int k = array.size() / 2;

        IntArray[] buckets = new IntArray[k];
        for (int i = 0; i < buckets.length; i++) {
            comparisons++;
            buckets[i] = new IntArray();
        }

        for (int i = 0; i < array.size(); i++) {
            int bi = (int) ((array.get(i) - min) / ((max - min + 1) / (double) k));
            if (!up) {
                bi = k - bi - 1;
            }
            moves++;
            comparisons++;
            buckets[bi].append(array.get(i));
        }

        printTrace(buckets[0]);
        for (int i = 1; i < buckets.length; i++) {
            printTrace(" | " + buckets[i]);
        }
        printTrace("\n");

        int index = 0;
        for (IntArray bucket : buckets) {
            for (int j = 0; j < bucket.size(); j++) {
                array.set(index++, bucket.get(j));
                moves++;
            }
        }

        insertionSort(array, up);
    }

    int getMin(IntArray array) {
        int min = array.get(0);
        comparisons++;
        for (int i = 1; i < array.size(); i++) {
            comparisons++;
            if (array.get(i) < min) {
                min = array.get(i);
            }
        }
        return min;
    }

    int getMax(IntArray array) {
        int max = array.get(0);
        comparisons++;
        for (int i = 1; i < array.size(); i++) {
            comparisons++;
            if (array.get(i) > max) {
                max = array.get(i);
            }
        }
        return max;
    }

    void insertionSort(IntArray array, boolean up) {
        for (int i = 1; i < array.size(); i++) {
            int key = array.get(i);
            moves++;
            int j = i - 1;
            while (j >= 0) {
                comparisons++;
                if (up ? array.get(j) <= key : array.get(j) >= key) {
                    break;
                }
                moves++;
                array.set(j + 1, array.get(j));
                j--;
            }
            moves++;
            array.set(j + 1, key);

            printTrace(array.toStringSplit(new int[]{i + 1}) + "\n");
        }
    }
}